
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Scanner;

/**
 * Solution to Letter Wheels
 */
public class wheels_vanb
{
    private static final long MOD = 982451653;
    
    /** The input. */
    private static Scanner sc;

    /** The output. */
    private static PrintStream ps;

    /**
     * Do it.
     */
    private void doit()
    {
        // Read the strings
        char s1[] = sc.next().toCharArray();
        char s2[] = sc.next().toCharArray();
        char s3[] = sc.next().toCharArray();
                
        // We can get n from the strings
        int n = s1.length;
        
        // p3 will be 3^n. We'll need it for our hash function.
        long p3 = 1L;
        for( int i=0; i<n; i++ ) p3 = (p3*3L) % MOD;
        
        // Compute a hash function. 
        // It's very simple. Let A=0, B=1, C=2. 
        // Then, consider the whole string as a base 3 number, modulo the MOD.
        long hash = 0L;
        for( int i=0; i<n; i++ ) hash = (3L*hash + (s3[i]-'A')) % MOD;
        
        // We'll create a HashMap of hash codes, 
        // and the positions in s3 where the associated string starts.
        HashMap<Long,List<Integer>> hashes = new HashMap<Long,List<Integer>>();
        for( int i=0; i<n; i++ )
        {
            // Addi to the list for this hash code (creating the list if it's not there yet)
            List<Integer> list = hashes.get( hash );
            if( list==null ) 
            {
                list = new ArrayList<Integer>();
                hashes.put( hash, list );
            }
            list.add( i );
            
            // This is the hash code for the next string along.
            // It's basically removing s3[i] from the beginning and adding it to the end.
            // That's where the p3 comes in. 
            hash = (3L*hash + (1l - p3 + MOD)*(s3[i]-'A')) % MOD;
        }
        
        // ok[k] is true if we can rotate wheel 2 (left) by k and have mismatches at every position
        boolean ok[] = new boolean[n];
        Arrays.fill( ok, true );
        for( int i=0; i<n; i++ ) for( int j=0; j<n; j++ ) if( s1[i]==s2[j] ) ok[(j-i+n)%n] = false;
              
        int best = Integer.MAX_VALUE;
        for( int k=0; k<n; k++ ) if( ok[k] && (k<best || n-k<best) )
        {
            // Here's a hash code for the target substring we'll look for in wheel 3
            hash = 0L;
            for( int i=0; i<n; i++ )
            {
                // t is whichever letter s1[i] and s2[i+k] aren't
                char t = (char)('A'+ ( ( (s1[i]-'A') + (s2[(i+k)%n]-'A') ) ^ 3 ) );
                
                // Build a hash code for this target string
                hash = (3L*hash + (t-'A')) % MOD;
            }
            
            // See if this hash code occurs in s3
            if( hashes.get( hash )!=null )
            {
                for( int p : hashes.get( hash ) )
                {
                    // We can always hold one of the wheels still
                    // What if it's wheel 1?
                    // We rotated wheel 2 by k and wheel 3 by p
                    // if we rotate a wheel left by x, then that's the same as rotating right by n-x
                    // So, we need to take the smaller of x and n-x
                    int answer1 = Math.min( k, n-k ) + Math.min( p, n-p );
                    if( answer1<best ) best = answer1;
                    
                    // What if it's wheel 2? 
                    // Wheel 1 then moves by k (in the other direction)
                    // Wheel 3 moves by p-k
                    int p2 = (p-k+n)%n;
                    int answer2 = Math.min( k, n-k ) + Math.min( p2, n-p2 );
                    if( answer2<best ) best = answer2;            
                    
                    // What if it's wheel 3?
                    // Wheel 2 then moves by p (in the other direction)
                    // Wheel 1 moves by k-p
                    int k3 = (k-p+n)%n;
                    int answer3 = Math.min( k3, n-k3 ) + Math.min( p, n-p );
                    if( answer3<best ) best = answer3;   
                }
            }
        }
        
        ps.println( best>n ? -1 : best );
    }

    /**
     * The main method.
     *
     * @param args the arguments (unused)
     */
    public static void main( String[] args )
    {
        sc = new Scanner( System.in );
        ps = System.out;
        new wheels_vanb().doit();
    }

}
